/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: RobertsVIIImporter.java,v 25b70b8a2daa 2010/08/23 14:48:42 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.birds.roberts;

import it.tidalwave.util.logging.Logger;
import javax.annotation.Nonnull;
import java.util.Scanner;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import org.openrdf.repository.Repository;
import it.tidalwave.bluebill.taxonomy.Taxonomy;
import it.tidalwave.bluebill.taxonomy.TaxonomyConcept;
import it.tidalwave.bluebill.taxonomy.birds.BirdTaxonomyImporter;
import it.tidalwave.bluebill.taxonomy.birds.MultiLanguageDisplayNameInitializer;
import it.tidalwave.util.EmptyInitializer;
import java.io.InputStream;
import static it.tidalwave.bluebill.taxonomy.elmo.ElmoTaxonomyVocabulary.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: RobertsVIIImporter.java,v 25b70b8a2daa 2010/08/23 14:48:42 fabrizio $
 *
 **********************************************************************************************************************/
public class RobertsVIIImporter extends BirdTaxonomyImporter
  {
    private static final String CLASS = RobertsVIIImporter.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Taxonomy run (@Nonnull final Repository repository,
                         @Nonnull final String taxonomyName)
      throws Exception
      {
        itisCodeMap.initialize();
        init(repository);
        
        final String root = ID_TAXONOMY_PREFIX + "Roberts/VII";
        final Taxonomy taxonomy = taxonomyManager.createTaxonomy("Roberts VII", root, repository);

        final String avesId = ID_TAXONOMY_PREFIX + "Roberts/VII" + "/Animalia/Chordata/Aves";
        final TaxonomyConcept aves = taxonomy.createTopConcept("Aves", avesId);
        setType(aves, classType);
        
        final InputStream is = getClass().getResourceAsStream("RobertsVII.csv");
        final BufferedReader br = new BufferedReader(new InputStreamReader(is, "iso-8859-1"));

        br.readLine();
        br.readLine();

        for (;;)
          {
            String s = br.readLine();

            if (s == null)
              {
                break;
              }

            s = s.trim();
            logger.fine(s);
            s = catalogueOfLifeTranslator.fixTypos(s);
//Order;Family;Latin Name;English Name;Afrikaans Name;German Name

            final Scanner scanner = new Scanner(s).useDelimiter(";");
            final String orderName = scanner.next().trim();
            final String familyName = scanner.next().trim();
            final String scientificName = scanner.next().trim();

            final Scanner scanner2 = new Scanner(scientificName).useDelimiter(" ");
            final String genusName = scanner2.next().trim();
            final String speciesName = scanner2.next().trim();
//            final String subSpeciesName = scanner2.hasNext() ? scanner2.next() : null;
            String englishName = scanner.next().trim();
            String afrikaansName = scanner.next().trim();
            String germanName = scanner.next().trim();

            logger.info("%s %s %s %s %s %s %s", orderName, familyName, orderName, speciesName, englishName, afrikaansName, germanName);

//            if (orderName.contains("Incertae Sedis") || familyName.contains("Incertae Sedis"))
//              {
//                continue;
//              }

            String path = "/Animalia/Chordata/Aves";
            path += "/" + orderName;
            final TaxonomyConcept order = findOrCreateSubConcept(aves, orderName, root, path, orderType, EmptyInitializer.<TaxonomyConcept>instance());
            path += "/" + familyName;
            final TaxonomyConcept family = findOrCreateSubConcept(order, familyName, root, path, familyType, EmptyInitializer.<TaxonomyConcept>instance());
            path += "/" + genusName;
            final TaxonomyConcept genus = findOrCreateSubConcept(family, genusName, root, path, genusType, EmptyInitializer.<TaxonomyConcept>instance());
            path += "/" + speciesName;
            final TaxonomyConcept species = findOrCreateSubConcept(genus, speciesName, root, path, speciesType,
                            new MultiLanguageDisplayNameInitializer("en", englishName, "de", germanName, "af", afrikaansName));
          }

        br.close();
        taxonUniqueIdManager.close();

        return taxonomy;
      }
  }
